# Needed to support type hint LISDQSProject for LISDQSConfiguration.get_project() method
# Convert type hint as str to avoid Class Not Found exceptions
from __future__ import annotations

from typing import List, Dict
from typing import TYPE_CHECKING
import os
import shutil

# To avoid cyclic import of LISDQSProject as this import is mainly used for type hinting
from qgis._core import QgsProject

from landsklim.lisdqs.utils import LandsklimUtils

if TYPE_CHECKING:
    from landsklim.lisdqs.lisdqs_project import LISDQSProject


from landsklim.lisdqs.lisdqs_analysis import LISDQSAnalysis
from landsklim.lisdqs.regressor import Regressor


class LISDQSConfiguration:
    """
    Represents a configuration.
    Configuration defines predictive variables for the interpolation model
    """

    def __init__(self, name: str, project: LISDQSProject):
        self._name: str = name
        self._lisdqs_project: LISDQSProject = project
        self._analysis: List[LISDQSAnalysis] = []

    def get_project(self) -> LISDQSProject:
        return self._lisdqs_project

    def add_analysis(self, analysis: LISDQSAnalysis):
        self._analysis.append(analysis)

    def get_analysis(self):
        return self._analysis

    def get_name(self) -> str:
        return self._name

    def __str__(self):
        return self.get_name()

    def to_string(self):
        return self.get_name()

    def delete_analysis(self, analysis: LISDQSAnalysis):
        analysis_path = analysis.get_path()
        if analysis in self._analysis:
            analysis.delete()
            self._analysis.remove(analysis)
        return analysis_path

    def to_json(self) -> Dict:
        state_dict: Dict = self.__dict__.copy()
        state_dict.pop('_lisdqs_project', None)
        return state_dict
